<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WPTC_File_Iterator{

	private $dir;
	private $wpdb;
	private $current_iterator_table;
	private $fs;
	private $file_base;
	private $staging_common;
	private $deep_dirs;

	public  function __construct(){
		$this->init_db();
		$this->current_iterator_table = new WPTC_Processed_iterator();
		$this->file_base = new Utils_Base();

		if (!defined('WPTC_BRIDGE') || !WPTC_BRIDGE) {
			$this->staging_common = new WPTC_Stage_Common();
		}

		$this->deep_dirs = array(
			WPTC_ABSPATH,
			WPTC_WP_CONTENT_DIR,
			WPTC_WP_CONTENT_DIR . '/' . WPTC_TEMP_DIR_BASENAME,
			WPTC_UPLOADS_DIR,
			WPTC_UPLOADS_DIR . '/'. WPTC_TEMP_DIR_BASENAME,
		);
	}

	private function init_db(){
		global $wpdb;
		$this->wpdb =$wpdb;
	}

	private function init_fs(){
		$this->fs = $this->staging_common->init_fs();
	}

	public function get_deep_dirs(){
		return $this->deep_dirs;
	}

	private function get_db_backup_file(){
		if (!is_any_ongoing_wptc_backup_process()) {
			return ;
		}

		$files_obj = $this->get_files_obj_by_path(WPTC_UPLOADS_DIR . '/'. WPTC_TEMP_DIR_BASENAME);
		$this->add_dir_list($files_obj);
	}

	public function get_folders(){

		if (defined('WPTC_BRIDGE') || !apply_filters('is_auto_backup_running_wptc', '')) {
			return $this->scan_entire_site();
		}

		if( defined('WPTC_BRIDGE') || $this->get_auto_update_folders() === 'normal_method'){
			return $this->scan_entire_site();
		}

		$this->get_db_backup_file();
		$this->save_dir_list();
	}

	private function scan_entire_site(){
		$this->get_root_dir_folders();
		$this->get_wp_content_dir_folders();
		$this->get_uploads_dir_folders();
		$this->get_db_backup_file();
		$this->save_dir_list();
		$this->save_deep_dir_list();
	}

	private function get_auto_update_folders(){
		$dirs = apply_filters('add_auto_backup_record_to_backup_wptc', '');

		if(empty($dirs)){
			return false;
		}

		$abspath = wptc_remove_trailing_slash(WPTC_ABSPATH);

		//if abspath is in the list then have to search for entire site so follow the normal method
		if(in_array($abspath, $dirs) !== false ){
			return 'normal_method';
		}

		foreach ($dirs as $dir) {
			$this->dir[] = $dir;
		}

		return true;
	}

	public function get_root_dir_folders(){
		$files_obj = $this->get_files_obj_by_path(WPTC_ABSPATH);
		$this->add_dir_list($files_obj);
	}

	public function get_wp_content_dir_folders(){
		$files_obj = $this->get_files_obj_by_path(WPTC_WP_CONTENT_DIR);
		$this->add_dir_list($files_obj);
	}

	public function get_uploads_dir_folders(){
		$files_obj = $this->get_files_obj_by_path(WPTC_UPLOADS_DIR);
		$this->add_dir_list($files_obj);
	}

	private function add_dir_list($files_obj){
		foreach ($files_obj as $key => $file_obj) {

			$file = $file_obj->getPathname();

			if (!wptc_is_dir($file)) {
				/// $this->files[] = $file;
			} else {
				$this->dir[] = $file;
			}
		}
	}

	public function save_dir_list(){
		dark_debug($this->dir, '--------$this->dir--------');
		$qry = '';

		foreach ($this->dir as $dir) {
			if (in_array($dir, $this->deep_dirs)) {
				dark_debug($dir, '--------$dir ignore this one--------');
				continue;
			}
			$qry .= empty($qry) ? "('" : ",('" ;
			$qry .= wp_normalize_path($dir) . "', '0')";

		}
		$this->insert_into_iterator_process($qry);
	}

	public function save_deep_dir_list(){
		dark_debug($this->deep_dirs, '--------$this->files--------');

		$qry = '';
		foreach ($this->deep_dirs as $dir) {

			$qry .= empty($qry) ? "('" : ",('" ;
			$qry .= wp_normalize_path($dir) . "', '0')";

		}

		$this->insert_into_iterator_process($qry);
	}

	private function insert_into_iterator_process($qry){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		$sql = "insert into " . $this->wpdb->base_prefix . "wptc_processed_iterator ( name, count  ) values $qry";
		dark_debug($sql, '--------$sql--------');
		$result = $this->wpdb->query($sql);
		dark_debug($result, '--------$result--------');
	}


	public function get_files_obj_by_path($path, $recursive = false){

		$obj = array();

		if (empty($path)) {
			return $obj;
		}

		$path = rtrim($path, '/');

		$path = wp_normalize_path($path);

		if (empty($path)) {
			return $obj;
		}

		$basename = basename($path);

		if ($basename == '..' || $basename == '.') {
			return $obj;
		}

		if (!is_readable($path)) {
			return $obj;
		}

		if($recursive){
			return new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path , RecursiveDirectoryIterator::SKIP_DOTS), RecursiveIteratorIterator::SELF_FIRST, RecursiveIteratorIterator::CATCH_GET_CHILD);
		}

		return new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path , RecursiveDirectoryIterator::SKIP_DOTS), RecursiveIteratorIterator::CATCH_GET_CHILD);
	}

	public function copy_dir($from, $to){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		$this->init_fs();
		$files = $this->get_files_obj_by_path($from, true);
		foreach ($files as $key => $file) {
			$pathname = $files->getPathname();

			$pathname = wp_normalize_path($pathname);

			if (wptc_is_dir($pathname)) {
				continue;
			}
			dark_debug($pathname, '--------$pathname--------');

			$to_file = str_replace($from, $to, $pathname);
			$to_file = wp_normalize_path($to_file);

			dark_debug($to_file, '--------$to_file--------');

			if (!$this->fs->exists(dirname($to_file))) {
				$this->same_server_mkdir(dirname($to_file));
			}
			$this->fs->copy($pathname, $to_file, true, FS_CHMOD_FILE);
		}
	}

	public function same_server_mkdir($path, $recursive = true){
		$path = wp_normalize_path($path);
		$this->file_base->createRecursiveFileSystemFolder($path, false, false);
	}
}