<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WPTC_Stage_Common{
	private $fs,
			$config,
			$processed_db;
	private $staging_id;

	public function __construct(){
		$this->config = WPTC_Factory::get('config');
		$this->processed_db = new WPTC_Processed_iterator();
	}

	public function init_fs(){
		global $wp_filesystem;
		if (!$wp_filesystem) {
			initiate_filesystem_wptc();
			if (empty($wp_filesystem)) {
				// $this->app_functions->die_with_json_encode(array("status" => "error", 'msg' => 'Could not initiate File system'));
				return false;
			}
		}
		$this->fs = $wp_filesystem;
		return $wp_filesystem;
	}

	public function init_db(){
		global $wpdb;
		$this->wpdb = $wpdb;
		return $wpdb;
	}

	public function get_table_data($table){
		$table_data = $this->processed_db->get_table($table);

		if ($table_data) {
			return array('offset' => $table_data->count, 'is_new' => false);
		}

		return array('offset' => 0, 'is_new' => true);
	}

	public function clone_table_structure($table, $new_table){
		$this->wpdb->query("DROP TABLE IF EXISTS `$new_table`");

		$is_cloned = $this->wpdb->query("CREATE TABLE `$new_table` LIKE `$table`");
		if (!$is_cloned) {
			$this->log('Creating table ' . $table . ' has been failed', 'staging_failed', $this->staging_id);
			dark_debug('Creating table ' . $table . ' has been failed.', '--------Failed-------------');
		} else {
			$this->log("Created table " . $table, 'staging_progress', $this->staging_id);
		}
	}

	public function clone_table_content($table, $new_table, $limit, $offset){
		while(true){
			$inserted_rows = 0;

			$inserted_rows = $this->wpdb->query(
				"insert `$new_table` select * from `$table` limit $offset, $limit"
			);

			dark_debug($inserted_rows, '---------$inserted_rows------------');

			if ($inserted_rows !== false) {
				if ($offset != 0) {
					$this->log('Copy database table: ' . $table . ' DB rows: ' . $offset , 'staging_progress', $this->staging_id); //create staging id
				}
				$offset = $offset + $inserted_rows;
				if ($inserted_rows < $limit) {
					$this->processed_db->update_iterator($table, -1); //Done
					break;
				}
				if(is_wptc_timeout_cut()){
					$this->processed_db->update_iterator($table, $offset);
					die_with_json_encode(array('status' => 'success', 'msg' => 'Cloning ' . $table . '(' . $offset . ')' , 'percentage' => 20));
				}
			} else {
				$this->processed_db->update_iterator($table, -1); //Done
				break;
				$this->log('Error: '.$this->wpdb->error.'inserting rows failed! Rows will be skipped. Offset: ' . $offset, 'staging_progress', $this->staging_id);
				dark_debug('Error: '.$this->wpdb->error.'Table ' . $new_table . ' has been created, but inserting rows failed! Rows will be skipped. Offset: ' . $offset , '--------Failed-------------');
			}
		}
	}

	public function init_staging_id(){
		$this->staging_id = $this->config->get_option('staging_id',true);
		if (empty($this->staging_id)) {
			$this->config->set_option('staging_id', time());
			$this->staging_id = $this->config->get_option('staging_id');
		}

		return $this->staging_id;
	}

	public function log($msg, $name, $id){
		return false;
		$this->logger->log('Copy database table: ' . $table . ' DB rows: ' . $offset , 'staging_progress', $this->staging_id);
	}

	public function replace_uri($old_url, $new_url, $old_file_path, $new_file_path, $table_prefix, $tables){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		$url_old_json = str_replace('"', "", json_encode($old_url));
		$url_new_json = str_replace('"', "", json_encode($new_url));
		$path_old_json = str_replace('"', "", json_encode($old_file_path));
		$path_new_json = str_replace('"', "", json_encode($new_file_path));


		$replace_list = array();

		array_push($replace_list,
				array('search' => $old_url,			 'replace' => $new_url),
				array('search' => $old_file_path,			 'replace' => $new_file_path),
				array('search' => $url_old_json,				 'replace' => $url_new_json),
				array('search' => $path_old_json,				 'replace' => $path_new_json),
				array('search' => urlencode($old_file_path), 'replace' => urlencode($new_file_path)),
				array('search' => urlencode($old_url),  'replace' => urlencode($new_url)),
				array('search' => rtrim(wp_normalize_path($old_file_path), '\\'), 'replace' => rtrim($new_file_path, '/'))
		);

		array_walk_recursive($replace_list, '_dupx_array_rtrim');

		dark_debug($replace_list, '---------------$replace_list -----------------');

		dark_debug($table_prefix, '---------------$table_prefix-----------------');
		if (empty($tables)) {
			$tables = $this->wpdb->get_results( 'SHOW TABLES LIKE "'.$table_prefix.'%"', ARRAY_N);
			dark_debug($tables, '---------------$tables replace_old_url inside-----------------');
		}
		dark_debug($tables, '---------------$tables replace_old_url-----------------');
		foreach ($tables as $key => $value) {
			dark_debug($value[0], '---------------$value replace_old_url-----------------');
			$this->replace_old_url_depth($replace_list, array($value[0]), true);
			dark_debug("Table ".$value[0]." URL content updated.", '-------------STATUS-------------------');
			unset($tables[$key]);
			if (count($tables) === 0) {
				$this->config->get_option('same_server_replace_old_url', true);
			} else {
				$this->config->set_option('same_server_replace_old_url_data', serialize($tables));
			}
			if(is_wptc_timeout_cut()){
				die_with_json_encode(array('status' => 'continue', 'msg' => 'Replacing links.', 'percentage' => 85));
			}
		}
	}

	public function replace_old_url_depth($list = array(), $tables = array(), $fullsearch = false) {
		$report = array(
			'scan_tables' => 0,
			'scan_rows' => 0,
			'scan_cells' => 0,
			'updt_tables' => 0,
			'updt_rows' => 0,
			'updt_cells' => 0,
			'errsql' => array(),
			'errser' => array(),
			'errkey' => array(),
			'errsql_sum' => 0,
			'errser_sum' => 0,
			'errkey_sum' => 0,
			'time' => '',
			'err_all' => 0
		);

		$walk_function = create_function('&$str', '$str = "`$str`";');


		if (is_array($tables) && !empty($tables)) {

			foreach ($tables as $table) {
				$report['scan_tables']++;
				$columns = array();
				$fields = $this->wpdb->get_results('DESCRIBE ' . $table); //modified
				foreach ($fields as $key => $column) {
					$columns[$column->Field] = $column->Key == 'PRI' ? true : false;
				}
				$row_count =  $this->wpdb->get_var("SELECT COUNT(*) FROM `{$table}`");
				if ($row_count == 0) {
					continue;
				}
				$page_size = $this->config->get_option('internal_staging_deep_link_limit');
				if (empty($page_size)) {
					$page_size = WPTC_STAGING_DEFAULT_DEEP_LINK_REPLACE_LIMIT; //fallback to default value
				}
				$offset = ($page_size + 1);
				$pages = ceil($row_count / $page_size);
				$colList = '*';
				$colMsg  = '*';
				if (! $fullsearch) {
					$colList = $this->get_text_columns($table);
					if ($colList != null && is_array($colList)) {
						array_walk($colList, $walk_function);
						$colList = implode(',', $colList);
					}
					$colMsg = (empty($colList)) ? '*' : '~';
				}

				if (empty($colList)) {
					continue;
				}
				$prev_table_data = $this->same_server_deep_link_status($table);
				if (!$prev_table_data) {
					$prev_table_data = 0;
				}
				//Paged Records
				for ($page = $prev_table_data; $page < $pages; $page++) {
					$current_row = 0;
					$start = $page * $page_size;
					$end   = $start + $page_size;
					$sql = sprintf("SELECT {$colList} FROM `%s` LIMIT %d, %d", $table, $start, $offset);
					$data  = $this->wpdb->get_results($sql);
					if (empty($data)){
						$scan_count = ($row_count < $end) ? $row_count : $end;
					}

				foreach ($data as $key => $row) {

						$report['scan_rows']++;
						$current_row++;
						$upd_col = array();
						$upd_sql = array();
						$where_sql = array();
						$upd = false;
						$serial_err = 0;

						foreach ($columns as $column => $primary_key) {
							$report['scan_cells']++;
							$edited_data = $data_to_fix = $row->$column;
							$base64coverted = false;
							$txt_found = false;


							if (!empty($row->$column) && !is_numeric($row->$column)) {
								//Base 64 detection
								if (base64_decode($row->$column, true)) {
									$decoded = base64_decode($row->$column, true);
									if ($this->is_serialized($decoded)) {
										$edited_data = $decoded;
										$base64coverted = true;
									}
								}

								//Skip table cell if match not found
								foreach ($list as $item) {
									if (strpos($edited_data, $item['search']) !== false) {
										$txt_found = true;
										break;
									}
								}
								if (! $txt_found) {
									continue;
								}

								//Replace logic - level 1: simple check on any string or serlized strings
								foreach ($list as $item) {
									$edited_data = $this->recursive_unserialize_replace($item['search'], $item['replace'], $edited_data);
								}

								//Replace logic - level 2: repair serilized strings that have become broken
								$serial_check = $this->fix_serial_string($edited_data);
								if ($serial_check['fixed']) {
									$edited_data = $serial_check['data'];
								} else if ($serial_check['tried'] && !$serial_check['fixed']) {
									$serial_err++;
								}
							}

							//Change was made
							if ($edited_data != $data_to_fix || $serial_err > 0) {
								$report['updt_cells']++;
								//Base 64 encode
								if ($base64coverted) {
									$edited_data = base64_encode($edited_data);
								}
								$upd_col[] = $column;
								$upd_sql[] = $column . ' = "' . $this->wpdb->_real_escape($edited_data) . '"';
								$upd = true;
							}

							if ($primary_key) {
								$where_sql[] = $column . ' = "' . $this->wpdb->_real_escape($data_to_fix) . '"';
							}
						}

						if ($upd && !empty($where_sql)) {

							$sql = "UPDATE `{$table}` SET " . implode(', ', $upd_sql) . ' WHERE ' . implode(' AND ', array_filter($where_sql));
							$result = $this->wpdb->query($sql);

							if ($result) {
								if ($serial_err > 0) {
									$report['errser'][] = "SELECT " . implode(', ', $upd_col) . " FROM `{$table}`  WHERE " . implode(' AND ', array_filter($where_sql)) . ';';
								}
								$report['updt_rows']++;
							}
						} elseif ($upd) {
							$report['errkey'][] = sprintf("Row [%s] on Table [%s] requires a manual update.", $current_row, $table);
						}
					}
					if(is_wptc_timeout_cut()){
						$this->config->set_option('same_server_replace_url_multicall_status', serialize(array($table =>($page+1))));
						dark_debug(array(), '---------------DEEP LINK NESTED TIMEOUT-----------------');
						dark_debug(array('table' => $table, 'start' => $start, 'offset' => $offset , 'page' =>($page+1)), '---------------DEEP LINK NESTED TIMEOUT data-----------------');
						die_with_json_encode(array('status' => 'continue', 'msg' => 'Replacing links - '. $table . '(' . $offset . ')' , 'percentage' => 40));
					}

				}

				if ($upd) {
					$report['updt_tables']++;
				}
			}
		}

		$report['errsql_sum'] = empty($report['errsql']) ? 0 : count($report['errsql']);
		$report['errser_sum'] = empty($report['errser']) ? 0 : count($report['errser']);
		$report['errkey_sum'] = empty($report['errkey']) ? 0 : count($report['errkey']);
		$report['err_all'] = $report['errsql_sum'] + $report['errser_sum'] + $report['errkey_sum'];
		return $report;
	}

	private function get_text_columns($table) {

		$type_where  = "type NOT LIKE 'tinyint%' AND ";
		$type_where .= "type NOT LIKE 'smallint%' AND ";
		$type_where .= "type NOT LIKE 'mediumint%' AND ";
		$type_where .= "type NOT LIKE 'int%' AND ";
		$type_where .= "type NOT LIKE 'bigint%' AND ";
		$type_where .= "type NOT LIKE 'float%' AND ";
		$type_where .= "type NOT LIKE 'double%' AND ";
		$type_where .= "type NOT LIKE 'decimal%' AND ";
		$type_where .= "type NOT LIKE 'numeric%' AND ";
		$type_where .= "type NOT LIKE 'date%' AND ";
		$type_where .= "type NOT LIKE 'time%' AND ";
		$type_where .= "type NOT LIKE 'year%' ";

		$result = $this->wpdb->get_results("SHOW COLUMNS FROM `{$table}` WHERE {$type_where}", ARRAY_N);
		if (empty($result)) {
			return null;
		}
		$fields = array();
		if (count($result) > 0 ) {
			foreach ($result as $key => $row) {
				$fields[] = $row['Field'];
			}
		}

		$result =  $this->wpdb->get_results("SHOW INDEX FROM `{$table}`", ARRAY_N);
		if (count($result) > 0) {
			foreach ($result as $key => $row) {
				$fields[] = $row['Column_name'];
			}
		}

		return (count($fields) > 0) ? $fields : null;
	}

	private function same_server_deep_link_status($table){
		dark_debug(array(), '---------------same_server_deep_link_status-----------------');
		$data = $this->config->get_option('same_server_replace_url_multicall_status');
		dark_debug($data, '---------------$data-----------------');
		if (empty($data)) {
			return false;
		}

		$unserialized_data = @unserialize($data);
		dark_debug($unserialized_data, '---------------$unserialized_data-----------------');
		if (empty($unserialized_data)) {
			return false;
		}
		if(!isset($unserialized_data[$table])){
			return false;
		}

		return $unserialized_data[$table];
	}

	private function recursive_unserialize_replace($from = '', $to = '', $data = '', $serialised = false) {
		try {
			if (is_string($data) && ($unserialized = @unserialize($data)) !== false) {
				$data = $this->recursive_unserialize_replace($from, $to, $unserialized, true);
			} else if (is_array($data)) {
				$_tmp = array();
				foreach ($data as $key => $value) {
					$_tmp[$key] = $this->recursive_unserialize_replace($from, $to, $value, false);
				}
				$data = $_tmp;
				unset($_tmp);
			} else if (is_object($data)) {

				$_tmp = $data;
				$props = get_object_vars( $data );
				foreach ($props as $key => $value) {
					$_tmp->$key = $this->recursive_unserialize_replace( $from, $to, $value, false );
				}
				$data = $_tmp;
				unset($_tmp);
			} else {
				if (is_string($data)) {
					$data = str_replace($from, $to, $data);
				}
			}

			if ($serialised)
				return serialize($data);

		} catch (Exception $error){

		}
		return $data;
	}

	private function fix_serial_string($data) {
		$result = array('data' => $data, 'fixed' => false, 'tried' => false);
		if (preg_match("/s:[0-9]+:/", $data)) {
			if (!$this->is_serialized($data)) {
				$regex = '!(?<=^|;)s:(\d+)(?=:"(.*?)";(?:}|a:|s:|b:|d:|i:|o:|N;))!s';
				$serial_string = preg_match('/^s:[0-9]+:"(.*$)/s', trim($data), $matches);
				//Nested serial string
				if ($serial_string) {
					$inner = preg_replace_callback($regex, array($this, 'fix_string_callback'), rtrim($matches[1], '";'));
					$serialized_fixed = 's:' . strlen($inner) . ':"' . $inner . '";';
				} else {
					$serialized_fixed = preg_replace_callback($regex, array($this, 'fix_string_callback'), $data);
				}
				if ($this->is_serialized($serialized_fixed)) {
					$result['data'] = $serialized_fixed;
					$result['fixed'] = true;
				}
				$result['tried'] = true;
			}
		}
		return $result;
	}

	public function fix_string_callback($matches) {
		return 's:'.strlen(($matches[2]));
	}

	private function is_serialized($data){
		return is_serialized_string( $data );
	}

	public function modify_wp_config($path, $site_url, $new_site_url, $new_prefix){
		if ($this->wpdb->base_prefix == $new_prefix) {
			dark_debug(array(), '--------Do not modify processing site site--------');
			return false;
		}
		$path = $path . 'wp-config.php';
		$content = file_get_contents($path);
		if ($content) {
			$content = str_replace('$table_prefix', '$table_prefix = \'' . $new_prefix . '\';//', $content); // replace table prefix
			$content = str_replace($site_url, $new_site_url , $content); // replace any url
			$content = str_replace('define(\'DB_NAME\'', 'define(\'DB_NAME\', \'' . $this->wpdb->dbname . '\');//', $content);
			$content = str_replace('define(\'DB_USER\'', 'define(\'DB_USER\', \'' . $this->wpdb->dbuser . '\');//', $content);
			$content = str_replace('define(\'DB_PASSWORD\'', 'define(\'DB_PASSWORD\', \'' . $this->wpdb->dbpassword . '\');//', $content);
			$content = str_replace('define(\'DB_HOST\'', 'define(\'DB_HOST\', \'' . $this->wpdb->dbhost . '\');//', $content);
			if (is_multisite()) {
				$staging_args    = parse_url($new_site_url);
				$staging_path  = rtrim($staging_args['path'], "/"). "/";
				$content = str_replace('define(\'PATH_CURRENT_SITE\'', 'define(\'PATH_CURRENT_SITE\', \'' . $staging_path . '\');//', $content);
			}
			$content = $this->remove_unwanted_data_from_wp_config($content);

			$content = $this->remove_unwanted_comment_lines($content, $is_wp_config = true);
			if (FALSE === file_put_contents($path, $content)) {
				$this->log($path . ' is not readable.', 'staging_progress', $this->staging_id);
				dark_debug(array(), '---------WP CONFIG NOT WRITABLE------------');
			} else{
				$this->log('wp-config has been successfully modified!', 'staging_progress', $this->staging_id);
				dark_debug(array(), '---------WP CONFIG REWROTE SUCCESSFULLY------------');
			}
		} else {
			$this->log($path . ' is not readable.', 'staging_progress', $this->staging_id);
			dark_debug($path . ' is not readable.', '---------FAILED------------');
		}
	}

	private function remove_unwanted_data_from_wp_config($content){
		$unwanted_words_match = array("WP_SITEURL", "WP_HOME");
		foreach ($unwanted_words_match as $words) {
			$replace_match = '/^.*' . $words . '.*$(?:\r\n|\n)?/m';
			$content = preg_replace($replace_match, '', $content);
		}

		return $content;
	}

	public function remove_unwanted_comment_lines($content, $is_wp_config = false){
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		$lines = explode("\n", $content);
		dark_debug($lines, '--------$lines--------');
		if ($is_wp_config) {
			$remove_comment_lines = array('DB_NAME', 'DB_USER', 'DB_PASSWORD', 'DB_HOST', 'PATH_CURRENT_SITE', 'table_prefix');
		} else {
			$remove_comment_lines = array('Changed by WP Time Capsule');
		}
		foreach ($lines as $key => $line) {
			foreach ($remove_comment_lines as $comment_lines) {
				if(strpos($line, $comment_lines) !== false){
					$lines[$key] = substr($line, 0, strpos($line, "//"));
				}
			}
		}
		dark_debug($lines, '--------$lines--------');
		return implode("\n", $lines);
	}

	public function multi_site_db_changes($new_prefix, $new_site_url){
		$staging_args    = parse_url($new_site_url);
		$staging_path  = rtrim($staging_args['path'], "/"). "/";
		$live_args = parse_url(get_home_url());
		$live_path = rtrim($live_args['path'], "/")."/";

		//update site table
		$result = $this->wpdb->query(
			$this->wpdb->prepare(
				'update ' . $new_prefix . 'site set path = %s',
				$staging_path
			)
		);
		if (!$result) {
			$error = isset($this->wpdb->error) ? $this->wpdb->error : '';
			$this->log('modifying site table is failed.' . $error, 'staging_progress', $this->staging_id);
			dark_debug('modifying site table is failed. ' . $error, '--------FAILED----------');
		} else {
			$this->log('modifying site table is successfully done.', 'staging_progress', $this->staging_id);
			dark_debug('modifying site table is successfully done.', '--------SUCCESS----------');
		}

		//update blogs table
		$sql = "update " . $new_prefix . "blogs set path = replace(path, '".$live_path."', '".$staging_path."') where path like '%".$live_path."%'";
		// dark_debug($sql, '---------------$sql-----------------');
		$result = $this->wpdb->query($sql);
		if (!$result) {
			$error = isset($this->wpdb->error) ? $this->wpdb->error : '';
			$this->log('modifying blogs table is failed.' . $error, 'staging_progress', $this->staging_id);
			dark_debug('modifying blogs table is failed. ' . $error, '--------FAILED----------');
		} else {
			$this->log('modifying blogs table is successfully done.', 'staging_progress', $this->staging_id);
			dark_debug('modifying blogs table is successfully done.', '--------SUCCESS----------');
		}

	}
}