<?php
class WPTC_Restore_Download {

	const MAX_DOWNLOAD_FAILED_LIMIT = 30;

	private $config,
			$file_iterator,
			$processed_files,
			$output,
			$processed_file_count,
			$is_continue_from_email,
			$post_data,
			$common_include_files,
			$initialize,
			$get_cur_res_b_id,
			$cloud_repo,
			$current_iterator_table,
			$prepared_file_array,
			$backup_type,
			$restore_app_functions,
			$restore_id;


	public function __construct() {
		//create object for restore app common
		$this->init_restore_app_functions();

		//enable PHP errors
		$this->restore_app_functions->enable_php_errors();

		//set global starting time
		$this->restore_app_functions->start_request_time();

		//define restore constant to override other functions
		$this->restore_app_functions->define_constants();

		//restore needed files
		$this->include_files();

		//assume this is request from server
		set_server_req_wptc();

		//start database connections
		$this->restore_app_functions->init_db_connection();

		$this->restore_app_functions->init_other_objects();

		$this->restore_app_functions->init_other_functions();

		//support for wp-content outside
		//set_basename_wp_content_dir();

		//start sentry logs
		// $this->common_include_files->start_sentry_restore_logs();

		//check whether its init request
		$this->check_init_request();

		dark_debug(array(), "--------Bridge above------OK--");

		//Init WP File system
		$this->restore_app_functions->init_file_system();

		$this->config = WPTC_Factory::get('config');

		//Check for other operations
		$this->check_for_other_operations();

		$this->restore_app_functions->init_log_files();

		//setup restore related data
		$this->setup_restore();

		//Init restore process
		$this->initiate_restore();
	}

	private function init_restore_app_functions(){
		//common app functions for both ajax and tc-init
		require_once dirname(__FILE__). '/' ."wptc-restore-app-functions.php";
		$this->restore_app_functions = new WPTC_Restore_App_Functions();
	}

	private function include_files(){
		require_once dirname(__FILE__). '/' ."common_include_files.php";
		$this->common_include_files = new Common_Include_Files('wptc-ajax');
		$this->common_include_files->init();
	}

	private function check_for_other_operations(){
		global $wp_filesystem;

		$this->restore_id = $this->config->get_option('restore_action_id');

		if (isset($this->post_data['action']) && $this->post_data['action'] == 'reset_restore_settings' ) {
			WPTC_Base_Factory::get('Wptc_Backup_Analytics')->send_report_data($this->restore_id, 'RESTORE', 'FAILED');
			WPTC_Factory::get('Debug_Log')->wptc_log_now('Reset restore related settings', 'RESTORE');
			dark_debug(array(), '-----------Cme in-------------');
			$this->config->remove_garbage_files(array('is_restore' => true), true);
			reset_restore_related_settings_wptc($dont_delete_logs = true);
			$this->restore_app_functions->die_with_msg("RESET_SUCCESSFULLY");
		}

		if (isset($this->post_data['action']) && $this->post_data['action'] == 'get_last_php_error' ) {
			dark_debug(array(), '-----------Cme in get_last_php_error-------------');
			$error = wptc_fatal_error_hadler(1);
			dark_debug($error, '---------$error------------');
			$this->restore_app_functions->die_with_msg($error);
		}

		if (!is_any_ongoing_wptc_restore_process()) {
				$this->restore_app_functions->die_with_msg(array('error' => 'This link has been expired, please initiate the fresh restore !'));
		}
	}

	private function check_init_request(){
		$this->initialize = false;

		$this->post_data = $_POST;

		if (!empty($_POST['initialize'])) {
			$this->initialize = true;
		} else if (isset($_POST['data'])) {
			$this->post_data = $this->restore_app_functions->decode_request();
			if (!empty($_POST['initialize'])) {
				$this->initialize = true;
			}
		}
		dark_debug($this->initialize, '--------$this->initialize--------');
	}

	private function setup_restore($post_data = array()){
		$this->is_continue_from_email = false;
		if (!empty($this->post_data['continue'])) {
			$this->is_continue_from_email = true;
		}

		$this->file_iterator = new WPTC_File_Iterator();
		$this->current_iterator_table = new WPTC_Processed_iterator();
		$this->processed_files = WPTC_Factory::get('processed-restoredfiles', true);
		$this->config->set_option('recent_restore_ping', time());
		$this->cloud_repo = WPTC_Factory::get(DEFAULT_REPO);
		$this->output = $this->output ? $this->output : WPTC_Extension_Manager::construct()->get_output();
	}

	private function process_safe_for_write_check_options() {
		if (empty($this->post_data)){
			return ;
		}

		if(!isset($this->post_data['ignore_file_write_check'])) {
			return ;
		}

		global $wpdb;
		if($this->post_data['ignore_file_write_check'] == 0) { //Skip only selected files so continue from where we left.

			$this->config->set_option('live_files_to_restore_table', false);
			$this->config->set_option('not_safe_for_write_files', false);
			$this->config->set_option('check_is_safe_for_write_restore', 1);

		} else if($this->post_data['ignore_file_write_check'] == 1) { //Skip all the files and start restore.
			$this->config->set_option('live_files_to_restore_table', 0);
			$wpdb->query("TRUNCATE TABLE `" . $wpdb->base_prefix . "wptc_current_process`");
			$wpdb->query("TRUNCATE TABLE `" . $wpdb->base_prefix . "wptc_processed_iterator`");
			$this->config->set_option('check_is_safe_for_write_restore', 0);
			$this->config->set_option('restore_current_action', false);

		} else if($this->post_data['ignore_file_write_check'] == 2) { //Restart file checking once again because user may fixed those issues
			$wpdb->query("TRUNCATE TABLE `" . $wpdb->base_prefix . "wptc_current_process`");
			$wpdb->query("TRUNCATE TABLE `" . $wpdb->base_prefix . "wptc_processed_iterator`");
			$this->config->set_option('restore_current_action', false);
			$this->config->set_option('live_files_to_restore_table', 0);
			$this->config->set_option('check_is_safe_for_write_restore', 1);
			$this->config->set_option('not_safe_for_write_files', false);
		}
	}

	public function initiate_restore() {

		$this->config->set_option('wptc_profiling_start', microtime(true));

		$this->config->set_option('in_progress_restore', true);

		$this->process_safe_for_write_check_options();

		$this->continue_copy_from_bridge_if_already_started();

		$data = array();

		if (!empty($this->initialize) && empty($data)) {
			//get requested restore meta data
			$data = $this->config->get_option('restore_post_data');
			if (empty($data)) {
				$this->proper_restore_complete_exit(array('error' => 'Didnt get Files to Restore'));
			}
			$data = unserialize($data);
		}

		$files_to_restore = array();

		if (isset($data['files_to_restore'])) {
			$files_to_restore = $data['files_to_restore'];
		}

			if (!empty($data['cur_res_b_id']) && $data['cur_res_b_id'] != 'false') {
			//This is restore to point
			$cur_res_b_id = $data['cur_res_b_id'];

			if (empty($files_to_restore)) {
				$files_to_restore = array('0' => 1); //dummy
			}
		}

		if (!empty($files_to_restore)) {
			if (!empty($cur_res_b_id)) {
				//Restore to point
				//the current b_id will be used to determine the future old files which are to be restored to the prev restore point
				$this->config->set_option('cur_res_b_id', $cur_res_b_id);
				$this->config->set_option('in_restore_deletion', false);
				$this->config->set_option('unknown_files_delete', true);
				$this->config->set_option('selected_id_for_restore', $cur_res_b_id);
			} else {
				//Restore to selected files
				$this->config->set_option('cur_res_b_id', '');
				$this->config->set_option('selected_id_for_restore', $data['selectedID']);
				$this->config->set_option('unknown_files_delete', false);
			}

			//set some flags and get sql files if its restore to point and then continue the restore process
			$this->set_flags_and_add_database_file($files_to_restore);

		} else {

			//if there is a bridge process going on ; then dont do restore-download
			if ($this->config->get_option('is_bridge_process')) {
				$this->restore_app_functions->die_with_msg("wptcs_over_wptce");
			}

			$restore_result = $this->start_restore();
		}
	}

	private function continue_copy_from_bridge_if_already_started() {
		if ($this->config->get_option('is_bridge_process')) {
			if ($this->is_continue_from_email) {
				$this->restore_app_functions->die_with_msg(array('continue_from_email' => true));
			}
		}
	}

	public function set_flags_and_add_database_file($files_to_restore) {

		manual_debug_wptc('', 'startingRestoreNow');

		//This never be empty because we add some dummy date on 162th line even its empty
		if (empty($files_to_restore)) {
			return true;
		}

		$this->config->set_option('in_progress_restore', true);

		$this->config->set_option('got_files_list_for_restore_to_point', 0);
		$this->config->set_option('live_files_to_restore_table', 0);
		$this->config->set_option('recorded_files_to_restore_table', 0);
		$this->config->set_option('selected_files_temp_restore', 0);
		$this->config->set_option('selected_backup_type_restore', 0);
		$this->config->set_option('got_selected_files_to_restore', 0);
		$this->config->set_option('not_safe_for_write_files', 0);
		$this->config->set_option('recorded_this_selected_folder_restore', 0);
		$this->config->set_option('is_bridge_process', 0);

		WPTC_Factory::get('logger')->log(sprintf(__('Restore started on %s.', 'wptc'), date("l F j, Y", strtotime(current_time('mysql')))), 'restore_start', $this->restore_id);
		$time = ini_get('max_execution_time');

		WPTC_Factory::get('logger')->log(sprintf(__('Your time limit is %s and your memory limit is %s'),
			$time ? $time . ' ' . __('seconds', 'wptc') : __('unlimited', 'wptc'),
			ini_get('memory_limit')
		), 'restore_start', $this->restore_id);

		if (ini_get('safe_mode')) {
			WPTC_Factory::get('logger')->log(__("Safe mode is enabled on your server so the PHP time and memory limit cannot be set by the Restore process. So if your Restore fails it's highly probable that these settings are too low.", 'wptc'), 'restore_error', $this->restore_id);
		}

		$cur_res_b_id = $this->get_cur_res_b_id();

		dark_debug($cur_res_b_id, "--------cur_res_b_id--------");

		$processed_base = new WPTC_Processed_Files();

		//this truncates the processed_restored_files table
		$this->processed_files->truncate();

		if (empty($cur_res_b_id)) {
			//This conditional loop is for restoring single or selected files.

			$selected_restore_id = $this->config->get_option("selected_id_for_restore");

			//get backup type such as D(Scheduled backup) or M(Manual backup)
			$backup_type = $processed_base->backup_type_check($selected_restore_id);

			$this->config->set_option('selected_files_temp_restore', serialize($files_to_restore));
			$this->config->set_option('selected_backup_type_restore', $backup_type);

			$this->config->set_option('got_files_list_for_restore_to_point', 1); //resetting flag indicating no need of restore_to_point

			WPTC_Factory::get('logger')->log(__("Files prepared for Restoring.", 'wptc'), 'restore_error', $this->restore_id);
		} else {
			//This conditional loop is for restoring to a point. This will have only the sql file.
			$this->config->set_option('got_selected_files_to_restore', 1); //resetting flag indicating no need of restore_single

			$backup_type = $processed_base->backup_type_check($cur_res_b_id);
			dark_debug($backup_type, "-------Backuptype-------");

			//during restore to point we need to prepare and send the sql files separately by the following function.
			$files_to_restore = $this->processed_files->get_formatted_sql_file_for_restore_to_point_id($cur_res_b_id, $backup_type);

			dark_debug($files_to_restore, "--------got sql files--------");

			//Add only the sql files to queue to download.
			$this->processed_files->add_files_for_restoring($files_to_restore, $backup_type);
		}

		$this->restore_pre_check(); //since we are using manual ajax function
	}

	public function restore_pre_check() {
		if (!$this->config->get_option('is_running_restore')) {
			$this->config->set_option('is_running_restore', true);
			//After all setups here we start the restore process.
			$this->start_restore();
		} else {
			$this->restore_app_functions->die_with_msg("wptcs_over_wptce");
		}
	}

	public function start_restore() {

		$this->config->set_memory_limit();

		$logger = WPTC_Factory::get('logger');

		try {
			if (!$this->cloud_repo || !$this->cloud_repo->is_authorized($is_restore = true)) {
				$logger->log(__('Your '.DEFAULT_REPO.' account is not authorized yet.', 'wptc'), 'restore_error', $this->restore_id);
				$this->proper_restore_complete_exit(array('error' => 'Your '.DEFAULT_REPO.' account is not authorized yet.'));
			}

			//Adding files to queue and start dowloading the files
			$result = $this->add_files_and_start_download();

			dark_debug($result, "--------result restore path--------");

			if (is_array($result) && isset($result['error'])) {
				$this->config->set_option('last_process_restore', false);
				$this->proper_restore_complete_exit($result);
			}

			$logger->log(__('Restore complete.', 'wptc'), 'restore_complete', $this->restore_id);
			$logger->log(sprintf(__('A total of %s files were processed.', 'wptc'), $this->processed_file_count), 'restore_complete', $this->restore_id);
			$logger->log(sprintf(
				__('A total of %dMB of memory was used to complete this restore.', 'wptc'),
				(memory_get_usage(true) / 1048576)
			), 'restore_complete', $this->restore_id);

			$root = false;
			if (get_class($this->output) != 'WPTC_Extension_DefaultOutput') {
				$this->output = new WPTC_Extension_DefaultOutput();
				$root = true;
			}

			//check chunk is alive yet, if yes then send call again response else over response
			$is_chunk_alive = $this->chunked_download_check();

			dark_debug($is_chunk_alive, "--------is_chunk_alive--------");

			if (!$this->config->get_option('chunked') && !$is_chunk_alive) {
				//if chunked download is not going ; or if bridge copy is not going do this completion step.
				$this->proper_restore_complete_exit('wptcs_over_wptce');
			} else {
				$this->restore_app_functions->die_with_msg("wptcs_callagain_wptce");
			}
		} catch (Exception $e) {
			WPTC_Factory::get('Debug_Log')->wptc_log_now('Restore failed - '.$e->getMessage(), 'RESTORE');
			if ($e->getMessage() == 'Unauthorized') {
				$logger->log(__('The plugin is no longer authorized with Cloud Repo.', 'wptc'), 'restore_error', $this->restore_id);
			} else {
				$logger->log(__('A fatal error occured: ', 'wptc') . $e->getMessage(), 'restore_error', $this->restore_id);
			}

			$this->proper_restore_complete_exit(array('error' => $e->getMessage()));
		}
	}

	//Checking the chunked download is in progress or completed
	public function chunked_download_check() {
		global $wpdb;
		//$unfinished_downloads = $wpdb->get_var('SELECT COUNT(*) FROM `' . $wpdb->base_prefix . 'wptc_processed_restored_files` WHERE `uploaded_file_size`!=`offset`  AND `uploaded_file_size` > 4024000');
		$unfinished_downloads = $wpdb->get_var(' SELECT COUNT(*) FROM `' . $wpdb->base_prefix . 'wptc_processed_restored_files` WHERE `offset` > 0 ');
		if ($unfinished_downloads > 0) {
			return true;
		} else {
			return false;
		}
	}

	public function proper_restore_complete_exit($message) {
		$this->config->complete('restore');

		if (!empty($message) && is_array($message)) {
			$this->config->remove_garbage_files(array('is_restore' => true));
			$this->restore_app_functions->die_with_msg($message);
		}

		$this->restore_app_functions->die_with_msg($message);
	}

	public function add_files_and_start_download($file = null, $version = null, $site_abspath = null) {
		if (!$this->config->get_option('in_progress_restore') || $this->config->get_option('in_progress')) {
			//this means restore got completed so returning empty response will end the restore
			return;
		}

		//just normalized abspath
		$site_abspath = get_tcsanitized_home_path();

		dark_debug($site_abspath, "--------got site_abspath--------");

		//this variable holds all the files which are already restored along with some info.
		$this->processed_files = WPTC_Factory::get('processed-restoredfiles', true);

		$this->processed_file_count = $this->processed_files->get_file_count();

		dark_debug(array(), "--------before--------");

		//this is true for restore to point because we have made it true when we add sql files to the queue eariler
		$got_selected_files_to_restore = $this->config->get_option("got_selected_files_to_restore");

		if (!$got_selected_files_to_restore) {
			//This conditional loop is only for restoring selected files
			$this->process_add_selected_files_to_restore();
			WPTC_Factory::get('logger')->log(__("Filelist prepared for Restoring Selected Files.", 'wptc'), 'restore_process', $this->restore_id);
		}

		//if we didnt store all the restore to point files to restore table, then continue it
		$got_restore_file_list = $this->config->get_option('got_files_list_for_restore_to_point');
		if (!$got_restore_file_list) {
			//This conditional loop is only for restore to point
			WPTC_Factory::get('logger')->log(__("Starting to prepare the Filelist for Restoring.", 'wptc'), 'restore_process', $this->restore_id);

			$cur_res_b_id = $this->get_cur_res_b_id();

			$processed_base = new WPTC_Processed_Files();

			if (!empty($cur_res_b_id)) {
				//This conditional loop is for Restore to point.

				$live_files_to_restore_table = $this->config->get_option('live_files_to_restore_table'); //flag for is live_files added to db

				dark_debug($live_files_to_restore_table, "--------live_files_to_restore_table value--------");

				$backup_type = $processed_base->backup_type_check($cur_res_b_id);

				if (empty($live_files_to_restore_table)) {
					//get file iterator obj
					// $Mfile_arr = $this->file_iterator->get_files_obj_by_path($site_abspath, $recursive = 1);
					//with iterator filtering files such as delete future files and check is writable
					$this->iterator_files($backup_type);
				}

				$recorded_files_to_restore_table = $this->config->get_option('recorded_files_to_restore_table');

				if (empty($recorded_files_to_restore_table)) {
					$this->get_recorded_files_to_restore_table($backup_type);
				}

				WPTC_Factory::get('logger')->log(__("Filelist prepared for Restoring.", 'wptc'), 'restore_process', $this->restore_id);

				$this->config->set_option('got_files_list_for_restore_to_point', 1);
			}
		}

		$restore_queue = $this->processed_files->get_limited_restore_queue_from_base();

		//dark_debug($restore_queue, "--------restore_queue--------");
		dark_debug(count($restore_queue), "--------restore_queue_count--------");

		static $is_queue = 0;
		$is_queue = count($restore_queue);
		WPTC_Factory::get('Debug_Log')->wptc_log_now('Start downloading files from cloud', 'RESTORE');

		while ($is_queue) {
			foreach ($restore_queue as $key => $value) {
				$additional_data = array();
				$additional_data = (array) $value;

				$file = $additional_data['file'];
				$version = $additional_data['revision_id'];

				$current_processed_files = $uploaded_files = array();

				$processed_file = $this->processed_files->get_file($file);
				// dark_debug($processed_file, '---------$processed_file------------');
				if ((($processed_file && $processed_file->offset == 0 && $processed_file->uploaded_file_size < 4024000 && $processed_file->download_status != "notDone") || ($processed_file && $processed_file->offset >= $processed_file->uploaded_file_size && $processed_file->download_status != "notDone")) && ($processed_file->revision_id == $version)) {
					//if processed_file value is not null , then the file is already restored so it needs to be skipped; but in case of chunked restored we should not skip it until the whole file is restored
					dark_debug($processed_file, "--------already restored ya--------");
					$this->config->set_option('chunked', false);
					continue;
				}
				// dark_debug(array(), '---------Download check------------');
				// dark_debug($file, '--------$file------------');
				// $is_same_res = $this->restore_app_functions->is_file_hash_same($file, $processed_file->file_hash ,$value->uploaded_file_size, $value->mtime_during_upload);
				// dark_debug($is_same_res, '---------$is_same_res------------');
				// if ($is_same_res) {
				// 	//dark_debug($file, "--------avoided sssm--------");
				// 	$cloud_response = true;
				// } else {
				// 	// dark_debug($site_abspath, "--------site_abspath--------");
					// dark_debug($file, "--------file--------");
					// dark_debug($version, "--------version--------");
					// dark_debug($processed_file, "--------processed_file--------");
					// dark_debug($value, "--------value--------");
				$cloud_response = $this->output->drop_download($site_abspath, $file, $version, $processed_file, (array) $value);
					// dark_debug($cloud_response, "--------cloud_response--------");
				// }
				// dark_debug($cloud_response, '--------$cloud_response--------');

				//dark_debug($cloud_response, "--------below download--------");

				if (is_array($cloud_response) && isset($cloud_response['error']) || empty($cloud_response)) {

					if (empty($cloud_response)) {
						$cloud_response['error'] = 'Unknown error';
					}
					$cloud_response['file'] = $file;

					$this->restore_app_functions->log_data('files', $cloud_response);

					$counter = $this->get_download_failed_file_count();
					$counter++;

					if (self::MAX_DOWNLOAD_FAILED_LIMIT <= $counter) {
						$this->hard_reset_restore();
						$err_msg['failure_data'] = $this->restore_app_functions->get_failure_data();
						$err_msg['error'] = 'Error: Maximum download failed limit crossed!';
						dark_debug($err_msg, '--------$err_msg--------');
						WPTC_Base_Factory::get('Wptc_Backup_Analytics')->send_report_data($this->restore_id, 'RESTORE', 'FAILED');
						$this->restore_app_functions->die_with_msg($err_msg);
					}

					$this->update_download_failed_file_count($counter);

					//change cloud response to true for marking this file as download completed
					$this->update_downloaded_file($cloud_response = true, $file, $additional_data, $current_processed_files, $strict = true);

				} else if (!empty($result) && is_array($result) && isset($result['too_many_requests'])) {

					//Temporary fails in google drive
					dark_debug($result, "--------too_many_requests--------");
					WPTC_Factory::get('logger')->log(__("Limit reached during download : .", 'wptc'), 'restore_process', $this->restore_id);

				} else if ($cloud_response) {
					//update download status in db
					$this->update_downloaded_file($cloud_response, $file, $additional_data, $current_processed_files);
				}

				//check timeout and echo "wptcs_callagain_wptce`" for each download files
				$this->restore_app_functions->maybe_call_again_tc();
			}
			$restore_queue = $this->processed_files->get_limited_restore_queue_from_base();
			$is_queue = count($restore_queue);
		}
	}

	private function get_download_failed_file_count(){
		$count = $this->config->get_option('download_failed_file_counter');
		return empty($count) ? 0 : $count;
	}

	private function update_download_failed_file_count($counter){
		$this->config->set_option('download_failed_file_counter', $counter);
	}

	private function update_downloaded_file($cloud_response, $file, $additional_data, $current_processed_files, $strict = false){
		$response_arr = (array) $cloud_response;

		if (!empty($response_arr['chunked'])) {
			$this->config->set_option('is_running_restore', false);
		} else {
			$this->config->set_option('chunked', false);
		}

		//get more data on this file.
		$processed_file = $this->processed_files->get_file($file);

		if ( (
				(!empty($processed_file) && $processed_file->download_status == 'notDone')	||
				(!empty($processed_file) && $processed_file->download_status == 'done')
			 )
			 &&
			 (!($this->config->get_option('chunked') && $strict === false ))
			) {

			$additional_data['download_status'] = 'done';
			$current_processed_files[] = $additional_data;

			$this->processed_file_count++;

			//adding files to DB here
			$this->processed_files->add_files($current_processed_files); //manual
		}
	}

	private function hard_reset_restore(){
		$this->config->set_option('in_progress_restore', false);
		$this->config->set_option('in_progress_restore', false);
		$this->config->set_option('is_running_restore', false);
		//$this->config->set_option('last_process_restore', false);
		$this->config->set_option('chunked', false);

		reset_restore_related_settings_wptc($dont_delete_logs = true);
	}

	public function process_add_selected_files_to_restore() {

		$selected_files_temp_restore = $this->config->get_option('selected_files_temp_restore');

		$backup_type = $this->config->get_option('selected_backup_type_restore');

		if (!$selected_files_temp_restore && !$backup_type) {
			$this->config->set_option('got_selected_files_to_restore', 1);
			return true;
		}

		$files_to_restore_tmp = unserialize($selected_files_temp_restore);

		dark_debug($files_to_restore_tmp, "--------files_to_restore_tmp--------");

		foreach ($files_to_restore_tmp as $files_or_folders => $v) {
			if ($files_or_folders == 'files') {
				foreach ($v as $file_dets) {
					$this->restore_app_functions->check_and_record_not_safe_for_write($file_dets['file_name']);
				}
				$this->check_and_exit_if_safe_for_write_limit_reached();

				$this->processed_files->add_files_for_restoring($v, $backup_type);
			} else if ($files_or_folders == 'folders') {
				foreach ($v as $file_dets) {
					$recorded_this_selected_folder_restore = $this->config->get_option_arr_bool_compat('recorded_this_selected_folder_restore');
					if (is_array($recorded_this_selected_folder_restore) && in_array($file_dets['file_name'], $recorded_this_selected_folder_restore)) {
						continue;
					}
					$this->get_recorded_files_of_this_folder_to_restore_table($file_dets['file_name'], $file_dets['backup_id'], $backup_type);
				}
			}
		}
		$this->config->set_option('got_selected_files_to_restore', 1);
	}

	public function check_and_exit_if_safe_for_write_limit_reached($path = false, $offset = false) {
		//return true; //off for now

		$check_is_safe_for_write = $this->config->get_option('check_is_safe_for_write_restore');
		$not_safe_for_write_files = $this->config->get_encoded_not_safe_for_write_files();

		if (!$check_is_safe_for_write || empty($not_safe_for_write_files)) {
			return true;
		}

		if (defined('WPTC_RESTORE_FILES_NOT_WRITABLE_COUNT') && (count($not_safe_for_write_files) >= WPTC_RESTORE_FILES_NOT_WRITABLE_COUNT)) {
			if ($path !== false && $offset !== false) {
				$this->current_iterator_table->update_iterator($path, $offset);
			}
			$this->restore_app_functions->die_with_msg((array('not_safe_for_write_limit_reached' => $not_safe_for_write_files)));
		}
	}

	public function iterator_files($backup_type = '') {

		$this->backup_type = $backup_type;

		$current_action = $this->config->get_option('restore_current_action');
		$current_action = empty($current_action) ? false : $current_action ;

		dark_debug($current_action, '--------$current_action--------');

		switch ($current_action) {
			case false:
				$this->get_folders();
				break;
			case 'get_files_by_folders':
				$this->get_files_by_folders();
				break;
			case 'start_restore':
			default:
				break;
		}

		dark_debug(array(), '--------Came to end--------');

		$this->config->set_option('live_files_to_restore_table', 1);

	}

	private function get_folders(){
		$this->file_iterator->get_folders();
		$this->config->set_option('restore_current_action', 'get_files_by_folders');
		$this->get_files_by_folders();
	}

	private function get_files_by_folders(){

		$break = false;
		$loop = $temp_counter = 0;

		while(!$break){
			$dir_meta = $this->current_iterator_table->get_unfnished_folder();

			$deep_dirs = false;

			if (empty($dir_meta) || $dir_meta->count === -1) {
				$break = true;
				continue;
			}

			$path = wp_normalize_path($dir_meta->name);

			if( array_search($path, $this->file_iterator->get_deep_dirs()) !== false ){
				$deep_dirs = true;
			}

			if ($deep_dirs === false && $this->is_skip($path)) {
				$this->current_iterator_table->update_iterator($path, -1);
				continue;
			}

			if(wptc_is_dir($dir_meta->name)){
				$this->process_dir($dir_meta->name, $dir_meta->count, $temp_counter, $deep_dirs);
			} else {
				$this->process_file($dir_meta->name, $write_on_complete = true);
			}
		}
		$this->config->set_option('restore_current_action', 'start_restore');
		return;
	}

	private function process_dir($path, $offset, &$temp_counter, $deep_dirs){

		$counter = 0;
		$qry = '';

		if ($deep_dirs) {
			$files_obj = $this->file_iterator->get_files_obj_by_path($path, $recursive = false);
		} else {
			$files_obj = $this->file_iterator->get_files_obj_by_path($path, $recursive = true);
		}

		foreach ($files_obj as $file_obj) {
			if ($counter++ < $offset ) {
				continue;
			}

			$offset++;
			$file = $file_obj->getPathname();

			$file = wp_normalize_path($file);

			if ($deep_dirs && wptc_is_dir($file)){
				continue;
			}

			if ($this->is_skip($file)) {
				$this->check_timeout_iter_file($path, $temp_counter, $offset);
				continue;
			}

			$this->process_file($file, false, $offset);

			$this->check_timeout_iter_file($path, $temp_counter, $offset);
		}

		$this->current_iterator_table->update_iterator($path, -1);
	}

	private function process_file($file, $write_on_complete = false, &$offset = 0){
		$file = wp_normalize_path($file);

		if ($this->is_skip($file)) {
			if (!$write_on_complete) {
				return ;
			}
			$this->current_iterator_table->update_iterator($file, -1);
			return;
		}
		//Check file is writable if not then change the mode , if cannot change then add it to unable restore list
		$this->restore_app_functions->check_and_record_not_safe_for_write($file);

		//delete all the new files which comes after current restore point
		$this->delete_future_files_and_get_restorable_files($file, $offset);

		if ($write_on_complete) {
			$this->current_iterator_table->update_iterator($file, -1);
		}
	}


	private function check_timeout_iter_file($path, &$temp_counter, $offset){
		if ($temp_counter++ < 30) {
			return false;
		}

		$this->check_and_exit_if_safe_for_write_limit_reached($path, $offset);

		$break = $this->restore_app_functions->maybe_call_again_tc($return = true);
		if (!$break) {
			return false;
		}


		$this->processed_files->add_files_for_restoring($this->prepared_file_array, $this->backup_type);
		$this->current_iterator_table->update_iterator($path, $offset);
		$this->prepared_file_array = array();
		$this->restore_app_functions->die_with_msg("wptcs_callagain_wptce");
	}

	private function is_skip($file){
		$basename = basename($file);

		if ($basename == '.' || $basename == '..') {
			return true;
		}


		if (stripos($file, 'wp-time-capsule') !== false || stripos($file, 'tCapsule') !== false || strpos($file, 'error_log') !== false) {
			return true;
		}


		if (stripos($file, 'wptc-secret') !== false) {
			return true;
		}

		if (false !== stripos($file, '/tCapsule/backups/')) {
			return true;
		}

		if (false !== stripos($file, 'DE_cl')) {
			return true;
		}

		if (false !== stripos($file, 'wp-tcapsule-bridge')) {
			return true;
		}

		if (false !== stripos($file, 'wptc_staging_controller')) {
			return true;
		}

		if (WPTC_Base_Factory::get('Wptc_ExcludeOption')->is_excluded_file($file)) {
			return true;
		}

		return false;
	}

	public function delete_future_files_and_get_restorable_files($file, &$offset) {

		if (is_dir($file)) {
			return true;
		}

		// if($this->config->get_option('is_staging_running') && stripos($file, '.htaccess') !== false ){
		// 	return ; //do not restore .htaccess if its staging
		// }

		$file = $this->config->wp_filesystem_safe_abspath_replace($file);
		$file = $this->config->replace_to_original_abspath($file);
		$file = rtrim($file, '/');

		$cur_res_b_id = $this->get_cur_res_b_id();

		$value = $this->processed_files->get_most_recent_revision($file, $cur_res_b_id);

		//avoiding metadata to be downloaded.
		if (empty($value) || ((strpos($file, 'meta_data') !== false) && (strpos($file, 'wptc-secret') !== false))) {
			//we will delete files before some pre checks
			return $this->safe_unlink_file($file, $offset);
		}

		$this->prepared_file_array[$value[0]->revision_id] = array();
		$this->prepared_file_array[$value[0]->revision_id]['file_name'] = wp_normalize_path($file);
		$this->prepared_file_array[$value[0]->revision_id]['file_size'] = $value[0]->uploaded_file_size;
		$this->prepared_file_array[$value[0]->revision_id]['mtime_during_upload'] = $value[0]->mtime_during_upload;
		$this->prepared_file_array[$value[0]->revision_id]['g_file_id'] = $value[0]->g_file_id;
		$this->prepared_file_array[$value[0]->revision_id]['file_hash'] = $value[0]->file_hash;
		$this->prepared_file_array[$value[0]->revision_id]['backup_id'] = $cur_res_b_id;
	}

	private function get_cur_res_b_id(){

		if ($this->get_cur_res_b_id) {
			return $this->get_cur_res_b_id;
		}

		$this->get_cur_res_b_id = $this->config->get_option("cur_res_b_id");
		return $this->get_cur_res_b_id;
	}

	public function get_recorded_files_of_this_folder_to_restore_table($folder_name, $folder_res_id, $backup_type) {
		dark_debug_func_map(func_get_args(), "--- function " . __FUNCTION__ . "--------", WPTC_DEBUG_PRINT_ALL);

		$records_queue = $this->processed_files->get_limited_recorded_files_of_this_folder_from_base($folder_res_id, $folder_name);
		dark_debug($records_queue, "--------records_queue--------");

		$this->prepared_file_array = array();

		static $is_queue = 0;
		$is_queue = count($records_queue);

		while ($is_queue) {
			foreach ($records_queue as $k => $v) {
				$this->restore_app_functions->check_and_record_not_safe_for_write($v->file);

				$value = $this->processed_files->get_most_recent_revision($v->file, $folder_res_id);

				$this->prepared_file_array[$value[0]->revision_id] = array();
				$this->prepared_file_array[$value[0]->revision_id]['file_name'] = wp_normalize_path($v->file);
				$this->prepared_file_array[$value[0]->revision_id]['file_size'] = $value[0]->uploaded_file_size;
				$this->prepared_file_array[$value[0]->revision_id]['mtime_during_upload'] = $value[0]->mtime_during_upload;
				$this->prepared_file_array[$value[0]->revision_id]['g_file_id'] = $value[0]->g_file_id;
				$this->prepared_file_array[$value[0]->revision_id]['file_hash'] = $value[0]->file_hash;
				$this->prepared_file_array[$value[0]->revision_id]['backup_id'] = $folder_res_id;
			}

			// $this->check_and_exit_if_safe_for_write_limit_reached();
			$this->processed_files->add_files_for_restoring($this->prepared_file_array, $backup_type);

			$this->restore_app_functions->maybe_call_again_tc();

			//below two lines is just for refreshing while loop condition
			$records_queue = $this->processed_files->get_limited_recorded_files_of_this_folder_from_base($folder_res_id, $folder_name);
			$is_queue = count($records_queue);

		}

		$this->config->append_option_arr_bool_compat('recorded_this_selected_folder_restore', $folder_name);
	}

	public function safe_unlink_file($val, &$offset) {
		dark_debug(func_get_args(), "--------" . __FUNCTION__ . "--------");
		global $wp_filesystem;
		if (!$wp_filesystem) {
			initiate_filesystem_wptc();
			if (empty($wp_filesystem)) {
				send_response_wptc('FS_INIT_FAILED-024');
				return false;
			}
		}

		$fs_safe_dir = $this->config->wp_filesystem_safe_abspath_replace(dirname($val));
		dark_debug($fs_safe_dir, '--------$fs_safe_dir--------');
		$fs_safe_file_name = $fs_safe_dir . basename($val);
		dark_debug($fs_safe_file_name, '--------$fs_safe_file_name--------');

		$result = $wp_filesystem->delete($fs_safe_file_name);

		dark_debug($result, '--------$result--------');

		if ($result === false) {
			dark_debug(error_get_last(), '--------error_get_last()--------');
		} else {
			--$offset;
		}

		if ($fs_safe_dir == trailingslashit($wp_filesystem->abspath()) || $fs_safe_dir == trailingslashit(WPTC_ABSPATH)) {
			return false;
		}

		$result = $wp_filesystem->delete($fs_safe_dir);
		if ($result === true) {
			--$offset;
		}
	}

	public function get_recorded_files_to_restore_table($backup_type) {
		manual_debug_wptc('', 'startingGet_recorded_files_to_restore_table');

		$cur_res_b_id = $this->get_cur_res_b_id();

		$offset = $this->config->get_option('get_recorded_files_to_restore_table');

		$offset = empty($offset) ? 0 : $offset;

		//get restore required files from the processed tables
		$records_queue = $this->processed_files->get_limited_recorded_files_queue_from_base($cur_res_b_id, $offset);

		manual_debug_wptc('', 'afterGetttingLimitedRecords');

		$this->prepared_file_array = array();

		static $is_queue = 0;
		$is_queue = count($records_queue);

		while ($is_queue) {
			$offset++;
			foreach ($records_queue as $k => $v) {
				$value = $this->processed_files->get_most_recent_revision($v->file, $cur_res_b_id);

				// $this->restore_app_functions->check_and_record_not_safe_for_write($v->file);

				$is_same_hash = $this->restore_app_functions->is_file_hash_same($v->file, $value[0]->file_hash ,$value[0]->uploaded_file_size, $value[0]->mtime_during_upload);

				if ($is_same_hash) {
					dark_debug($v->file, '-----Hash does not modified so continue-----------');
					continue;
				}

				dark_debug($v->file, '-----This file changed-----------');

				$this->prepared_file_array[$value[0]->revision_id] 						= array();
				$this->prepared_file_array[$value[0]->revision_id]['file_name'] 			= wp_normalize_path($v->file);
				$this->prepared_file_array[$value[0]->revision_id]['file_size'] 			= $value[0]->uploaded_file_size;
				$this->prepared_file_array[$value[0]->revision_id]['mtime_during_upload'] = $value[0]->mtime_during_upload;
				$this->prepared_file_array[$value[0]->revision_id]['g_file_id'] 			= $value[0]->g_file_id;
				$this->prepared_file_array[$value[0]->revision_id]['file_hash'] 			= $value[0]->file_hash;
				$this->prepared_file_array[$value[0]->revision_id]['backup_id'] 			= $cur_res_b_id;
			}

			$this->check_and_exit_if_safe_for_write_limit_reached();

			manual_debug_wptc('', 'beforeAddingFilesForRestoring');

			if (!empty($this->prepared_file_array)) {
				//add files to restore processing table
				$this->processed_files->add_files_for_restoring($this->prepared_file_array, $backup_type);
			}

			manual_debug_wptc('', 'afterAddingFilesForRestoring');

			if($this->restore_app_functions->maybe_call_again_tc($return = true)){
				$this->config->set_option('get_recorded_files_to_restore_table', $offset);
				$this->restore_app_functions->die_with_msg("wptcs_callagain_wptce");
			}

			//resetting the array once again
			$this->prepared_file_array = array();

			//below two lines is just for refreshing while loop condition
			$records_queue = $this->processed_files->get_limited_recorded_files_queue_from_base($cur_res_b_id, $offset);
			$is_queue = count($records_queue);
		}

		$this->config->set_option('recorded_files_to_restore_table', 1);
	}
}

new WPTC_Restore_Download();